unit Schlange;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Variants, Messages;

type

  { TSchlange }

  TSchlange = class(TObject)
      FTeile: array[0..40*30-1] of TPoint;
      FAnzahlTeile: Word;
      FRichtung: Byte;
      FWurdeBewegt: boolean;
      FFarbe: TColor;
      Richtung: Byte;
      AnzahlTeile: Word;
      FCanvas: TCanvas;
      constructor Create;
      function PruefeObTot: boolean;
      function LiesTeil(index: Word): TPoint;
      function Farbe: TColor;
      function IsOnPoint(p: TPoint): boolean;
      property Teile[index: Word]: TPoint read LiesTeil;
      property Canvas: TCanvas read FCanvas write FCanvas;
      procedure Init;
      procedure Render;
      procedure Update;
      procedure WachseUm1;
      procedure AendereRichtung(NeueRichtung: integer);
  end;

implementation

{ TSchlange }

constructor TSchlange.Create;
begin
  FFarbe:= clWhite;
end;

function TSchlange.PruefeObTot: boolean;
var
  i: integer;
  index: word;
begin
  for index:= 1 to FAnzahlTeile do
  begin
    if (FTeile[0].x = FTeile[index].x) and (FTeile[0].y = FTeile[index].y) then
    begin
      Result:= true;
      Exit;
    end;
  end;

  Result:= false;
end;

function TSchlange.LiesTeil(index: Word): TPoint;
begin
  Result:= FTeile[index];
end;

function TSchlange.Farbe: TColor;
begin
  Result:= FFarbe;
end;

function TSchlange.IsOnPoint(p: TPoint): boolean;
var
  index: Word;
begin
  Result:= false;
  for index:= 0 to FAnzahlTeile - 1 do
  begin
    if(FTeile[index].x = p.x) and (FTeile[index].y = p.y) then
      Result:= true;
  end;
end;

procedure TSchlange.Init;
begin
  FAnzahlTeile:= 2;
  FTeile[0]:= Point(3, 3);
  FTeile[1]:= Point(2, 3);
  FTeile[2]:= Point(1, 3);

  FRichtung:= 3;
  FWurdeBewegt:= true;
end;

procedure TSchlange.Render;
var
  index: word;
  Startx, Starty: integer;
  Endex, Endey: integer;
begin
  FCanvas.Pen.Color:= Self.FFarbe;
  FCanvas.Brush.Color:= Self.FFarbe;

  for Index:= 0 to FAnzahlTeile do
  begin
    Startx:= FTeile[index].x * 16;
    Starty:= FTeile[index].y * 16;
    Endex:= FTeile[index].x * 16 + 16;
    Endey:= FTeile[index].y * 16 + 16;
    FCanvas.Rectangle(Startx, Starty, Endex, Endey);
  end;
end;

procedure TSchlange.Update;
var
  index: Word;
begin

  //Teile hinterher ziehen
  for index:= FAnzahlTeile downto 1 do
  begin
    FTeile[index]:= FTeile[index - 1];
  end;

  //Ablauf für das Verlassen des Spielfeldes und Wiedererscheinen auf der gegenüberliegenden Seite
  case FRichtung of
  0: //Bewegung nach oben
    begin
      FTeile[0].y:= FTeile[0].y - 1;
      if FTeile[0].y < 0 then
      begin
        FTeile[0].y:= 29;
        FWurdeBewegt:= true;
      end;
    end;

  1: //Bewegung nach unten
    begin
      FTeile[0].y:= FTeile[0].y + 1;
      if FTeile[0].y > 29 then
      begin
        FTeile[0].y:= 0;
        FWurdeBewegt:= true;
      end;
    end;

  2: //Bewegung nach links
    begin
      FTeile[0].x:= FTeile[0].x - 1;
      if FTeile[0].x < 0 then
      begin
        FTeile[0].x:= 39;
        FWurdeBewegt:= true;
      end;
    end;

  3: //Bewegung nach rechts
    begin
      FTeile[0].x:= FTeile[0].x + 1;
      if FTeile[0].x > 39 then
      begin
        FTeile[0].x:= 0;
        FWurdeBewegt:= true;
      end;
    end;
  end;
end;

procedure TSchlange.WachseUm1;
begin
  FAnzahlTeile:= FAnzahlTeile + 1;
end;

procedure TSchlange.AendereRichtung(NeueRichtung: integer);
begin
  case NeueRichtung of
  0: //Nach Oben
    begin
      if not Self.FRichtung = 1 then
      begin
        Self.FRichtung:= 0;
      end;
    end;

  1: //Nach Unten
    begin
      if not Self.FRichtung = 0 then
      begin
        Self.FRichtung:= 1;
      end;
    end;

2:  //Nach Links
    begin
      if not Self.FRichtung = 3 then
      begin
        Self.FRichtung:= 2;
      end;
    end;

3:  //Nach Rechts
    begin
      if not Self.FRichtung = 2 then
      begin
        Self.FRichtung:= 3;
      end;
    end;
  end;

  FRichtung:= NeueRichtung;
  FWurdeBewegt:= false;
end;

end.

